package nl.rubend.sudoku.internal

import nl.rubend.sudoku.internal.data.Grid
import nl.rubend.sudoku.internal.solver.gridStrategies.GridStrategy
import nl.rubend.sudoku.internal.solver.MainStrategy
import nl.rubend.sudoku.internal.solver.rowStrategies.ValidateStrategy
import kotlin.random.Random

class Generator(validateGridStrategies: List<GridStrategy>, val solveStrategy: MainStrategy, val size: Int, val maxRemove: Int=size*size, seed: Long = Random.nextLong()) {
    val validateStrategy: MainStrategy = MainStrategy(validateGridStrategies, listOf(ValidateStrategy()))
    val random=Random(seed)
    fun run(): Grid {
        var randomized = randomCell(Grid(size), 0, 0)!!
        if(!randomized.isSolved()) throw RuntimeException("Generator failed!")
        val randomPairs=(0 until size).flatMap { x->(0 until size).map { y-> x to y }}.shuffled(random)
        var removed=0
        randomPairs.forEach {(x,y) ->
            if(removed==maxRemove) return@forEach
            val temp=randomized.clone()
            temp.rows[x].cells[y].possibleValues=(1..size).toMutableList()
            try {
                if(solveStrategy.run(temp.clone())==null) return@forEach
                randomized = temp
                removed++
            } catch (_: Exception) {

            }
        }
        validateStrategy.getRows(randomized)
        return randomized
    }

    fun randomCell(grid: Grid, x: Int, y: Int): Grid? {
        val cell=grid.rows[x].cells[y]
        val notTried=(1..size).shuffled(random).toMutableList()
        while(notTried.isNotEmpty()) {
            cell.setValue(notTried.removeFirst())
            try {
                validateStrategy.run(grid)
            } catch (e: Exception) {
                continue
            }
            var x2=x+1
            var y2=y
            if(x2==size) {
                x2=0
                y2++
            }
            if(y2==size) return grid
            return randomCell(grid.clone(), x2, y2)?:continue
        }
        return null
    }
}
