package nl.rubend.sudoku.internal.solver

import nl.rubend.sudoku.internal.data.Grid
import nl.rubend.sudoku.internal.data.Row
import nl.rubend.sudoku.internal.solver.gridStrategies.GridStrategy
import nl.rubend.sudoku.internal.solver.rowStrategies.RowStrategy

class MainStrategy(private val gridStrategies: List<GridStrategy>, private val rowStrategies: List<RowStrategy>) {
    fun getRows(grid: Grid): List<Row> {
        val rows = gridStrategies.map { it.getRows(grid) }.flatten()
        rows.forEachIndexed { index, row->
            row.id=index
            row.cells.forEach {cell -> cell.rows.add(index) }
        }
        return rows
    }
    fun run(grid: Grid): Grid? {
        val rows = getRows(grid)
        while(true) {
            val before = grid.clone()
            rowStrategies.forEach { strategy ->
                repeat(strategy.iterations) { iteration ->
                    rows.forEach { row ->
                        strategy.doStrategy(row, iteration)
                    }
                }
            }
            grid.validate()
            if (before == grid) break
        }
        return if(grid.isSolved()) grid
        else {
//            println(grid)
//            println(grid.rows.map { it.cells.map { it.possibleValues } })
            null
        }
    }
}
