package nl.rubend.sudoku.webserver

import nl.rubend.sudoku.internal.Generator
import nl.rubend.sudoku.internal.solver.MainStrategy
import nl.rubend.sudoku.internal.solver.gridStrategies.*
import nl.rubend.sudoku.internal.solver.rowStrategies.*
import nl.rubend.sudoku.webserver.dto.GridDTO
import nl.rubend.sudoku.webserver.dto.RequestDTO
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("/api")
class TestServer {
    @GetMapping("/{seed}")
    fun generateWithSeed(@PathVariable seed: Long): GridDTO {
        val validateGridStrategies = listOf<GridStrategy>(
            BlockStrategy(),
            ColumnStrategy(),
            EveryRowStrategy(),
//            DiagonalStrategy(),
//            ExtraBlockStrategy(),
        )

        val solveRowStrategies = listOf(
            SimpleRemoveStrategy(),
            ComplicatedRemoveStrategy(2),
        )
        val solveGridStrategies = listOf<GridStrategy>(
            BlockStrategy(),
            ColumnStrategy(),
            EveryRowStrategy(),
//            DiagonalStrategy(),
//            ExtraBlockStrategy(),
        )


        val solveStrategy = MainStrategy(solveGridStrategies, solveRowStrategies)

        val grid = Generator(
            validateGridStrategies=validateGridStrategies,
            solveStrategy=solveStrategy,
            size=9,
            seed=seed
        ).run()
        return GridDTO(grid)
    }

    @PostMapping("/rows")
    fun getRows(@RequestBody request: RequestDTO): GridDTO {
        val grid = request.grid.toGrid()
        request.settings.getGenerator().solveStrategy.getRows(grid)
        return GridDTO(grid)
    }

    @PostMapping("/puzzle")
    fun getPuzzle(@RequestBody request: RequestDTO): GridDTO {
        return GridDTO(request.settings.getGenerator().run())
    }

    @PostMapping("/solve")
    fun solve(@RequestBody request: RequestDTO): GridDTO {
        val grid = getPuzzle(request).toGrid()
        return GridDTO(request.settings.getGenerator().solveStrategy.run(grid)!!)
    }
    @PostMapping("/canSolve")
    fun canSolve(@RequestBody request: RequestDTO): Boolean {
        val grid = request.grid.toGrid()
        return request.settings.getGenerator().solveStrategy.run(grid)!=null
    }
}
