package nl.rubend.sudoku.webserver.dto

import nl.rubend.sudoku.internal.Generator
import nl.rubend.sudoku.internal.solver.MainStrategy
import nl.rubend.sudoku.internal.solver.gridStrategyFactory
import nl.rubend.sudoku.internal.solver.rowStrategyFactory
import kotlin.random.Random

data class SettingsDTO(
    val validateGridStrategies: Map<String,Map<String,String>>,
    val solveRowStrategies: Map<String,Map<String,String>>,
    val solveGridStrategies: Map<String,Map<String,String>>,
    val seed: Long = Random.nextLong(),
    val size: Int = 9,
    val maxRemove: Int = size*size,
) {

    fun getGenerator(): Generator {
        val validateGridStrategiesList = validateGridStrategies.map { (strategyName, strategySettings) -> gridStrategyFactory(strategyName, strategySettings) }
        val solveRowStrategiesList = solveRowStrategies.map { (strategyName, strategySettings) -> rowStrategyFactory(strategyName, strategySettings) }
        val solveGridStrategiesList = solveGridStrategies.map { (strategyName, strategySettings) -> gridStrategyFactory(strategyName, strategySettings) }
        val solveStrategy = MainStrategy(solveGridStrategiesList, solveRowStrategiesList)

        return Generator(
            validateGridStrategies=validateGridStrategiesList,
            solveStrategy=solveStrategy,
            size=size,
            seed=seed,
            maxRemove=maxRemove,
        )
    }
}
